/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro                                  *
 *              http://picosa.narod.ru                                                          *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:       osa_tasks.h                                                                     *
 *                                                                                              *
 *  Compilers:  HT-PICC STD                                                                     *
 *              HT-PICC18 STD                                                                   *
 *              Microchip C18                                                                   *
 *              Microchip C30                                                                   *
 *                                                                                              *
 *  Programmer: Timofeev Victor                                                                 *
 *              osa@pic24.ru, testerplus@mail.ru                                                *
 *                                                                                              *
 *  Definition: Services for work with tasks                                                    *
 *                                                                                              *
 *  History:    26.02.2009  Bug fixed: OS_Task_Replace did not reserve created task by fncall   *
 *                          OS_ReserveTask renamed to OS_Task_Reserve                           *
 *              01.04.2009  All processor specific services for tasks (create, replace and      *
 *                          reserve) were moved into proc specific h-file in "port" folder      *
 *                                                                                              *
 ************************************************************************************************
 */



/************************************************************************************************
 *                                                                                              *
 *                                   T A S K S                                                  *
 *                                                                                              *
 ************************************************************************************************/


#define this_task   _OS_CurTask

/************************************************************************/
/*                                                                      */
/* Stop current task                                                    */
/*                                                                      */
/************************************************************************/

#define OS_Task_Stop()   OS_Task_Delete(this_task)


/************************************************************************/
/*                                                                      */
/* Delete task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Delete(pTask)                                           \
    {                                                                   \
        (pTask)->State.bEnable = 0;                                     \
        if ((&(pTask)) == (&_OS_CurTask))                               \
        {                                                               \
            _OS_Return();                                               \
        }                                                               \
    }


/************************************************************************/
/*                                                                      */
/* Pause task                                                           */
/*                                                                      */
/************************************************************************/

#define OS_Task_Pause(pTask)                                            \
    {                                                                   \
        (pTask)->State.bPaused = 1;                                     \
        if ((&(pTask)) == (&_OS_CurTask))                               \
        {                                                               \
            _OS_Return();                                               \
        }                                                               \
    }

/************************************************************************/
/*                                                                      */
/* Continue paused task                                                 */
/*                                                                      */
/************************************************************************/

#define OS_Task_Continue(pTask)          (pTask)->State.bPaused = 0



/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

// See proc.spec. h-file in "port" folder.



/****************************************************************************/
/*                                                                          */
/* Create task                                                              */
/*                                                                          */
/****************************************************************************/

// See proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_CreateP(priority, TaskAddr)      _OS_Task_Create(priority, TaskAddr)

// If task created indirrectly then it is nessesary to insert in
// the main() servise OS_ReserveTasks
// See proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Get/set current task priority                                        */
/*                                                                      */
/************************************************************************/

#define OS_Task_GetPriority(pTask)                   (pTask)->State.Priority
#define OS_Task_SetPriority(pTask, new_priority)     (pTask)->State.Priority = new_priority


/************************************************************************/
/*                                                                      */
/* Get current task's descriptor address                                */
/*                                                                      */
/************************************************************************/

#define OS_GetCurTask()                 this_task


/************************************************************************/
/*                                                                      */
/* Check for task is paused                                             */
/*                                                                      */
/************************************************************************/

#define OS_Task_IsPaused(pTask)          (pTask)->State.bPaused



















